package com.example.backend.viewed.api;

import java.util.List;
import java.util.ArrayList;

import org.modelmapper.ModelMapper;
import org.springframework.security.core.Authentication;
import org.springframework.security.core.context.SecurityContextHolder;
import org.springframework.stereotype.Controller;
import org.springframework.ui.Model;
import org.springframework.web.bind.annotation.GetMapping;
import org.springframework.web.bind.annotation.PathVariable;
import org.springframework.web.bind.annotation.PostMapping;
import org.springframework.web.bind.annotation.RequestMapping;
import org.springframework.web.bind.annotation.RequestParam;

import com.example.backend.core.configurations.Constants;
import com.example.backend.favorites.model.FavoriteEntity;
import com.example.backend.favorites.service.FavoriteService;
import com.example.backend.viewed.model.ViewedEntity;
import com.example.backend.viewed.service.ViewedService;
import com.example.backend.movies.api.MovieDTO;
import com.example.backend.movies.api.MovieUserDTO;
import com.example.backend.movies.model.MovieEntity;
import com.example.backend.movies.service.MovieService;
import com.example.backend.users.model.UserEntity;
import com.example.backend.users.service.UserService;

@Controller
@RequestMapping(ViewedController.URL)
public class ViewedController {

    public static final String URL = "/viewed";
    private static final String FAVORITE_VIEW = "viewed";
    private static final String PAGE_ATTRIBUTE = "page";

    private final FavoriteService favoriteService;
    private final UserService userService;
    private final MovieService movieService;
    private final ModelMapper modelMapper;
    private final ViewedService viewedServise;

    public ViewedController(
            FavoriteService favoriteService, UserService userService,
            MovieService movieService, ModelMapper modelMapper, ViewedService viewedServise) {
        this.modelMapper = modelMapper;
        this.userService = userService;
        this.favoriteService = favoriteService;
        this.viewedServise = viewedServise;
        this.movieService = movieService;
    }

    private MovieDTO toDtoMovie(MovieEntity entity) {
        return modelMapper.map(entity, MovieDTO.class);
    }

    private List<MovieUserDTO> getListMovieUserDTOs(Integer categorieId, int page) {
        List<MovieDTO> movies = movieService.getAll(categorieId, page, Constants.DEFUALT_PAGE_SIZE)
                .stream()
                .map(this::toDtoMovie)
                .toList();
        List<MovieUserDTO> muDTO = new ArrayList<MovieUserDTO>();

        Authentication authentication = SecurityContextHolder.getContext().getAuthentication();
        Integer currentUserId = userService.getCurrentUserId(authentication.getName());

        List<FavoriteEntity> favs = favoriteService.getAll(currentUserId);
        List<ViewedEntity> viewes = viewedServise.getAll(currentUserId);

        for (MovieDTO movieDTO : movies) {
            MovieUserDTO newMovDto = new MovieUserDTO();
            newMovDto.setMovieDTO(movieDTO);
            newMovDto.setCountViewes(movieService.countView(movieDTO.getId()));

            for (FavoriteEntity favoriteEntity : favs) {
                if ((int) favoriteEntity.getMovie().getId() == (int) movieDTO.getId()) {
                    newMovDto.setFavorite(true);
                    break;
                }
                newMovDto.setFavorite(false);
            }

            for (ViewedEntity viewedEntity : viewes) {
                if ((int) viewedEntity.getMovie().getId() == (int) movieDTO.getId()) {
                    newMovDto.setViewed(true);
                    break;
                }
                newMovDto.setViewed(false);
            }

            muDTO.add(newMovDto);
        }
        return muDTO;
    }

    @GetMapping()
    public String getAll(Model model, @RequestParam(name = PAGE_ATTRIBUTE, defaultValue = "0") int page) {
        List<MovieUserDTO> muDTO = getListMovieUserDTOs(0, page);
        model.addAttribute("movies", muDTO);

        model.addAttribute(PAGE_ATTRIBUTE, page);
        return FAVORITE_VIEW;
    }

    @PostMapping("/changeFavStatus/{id}")
    public String changeFavStatus(@PathVariable(name = "id") Integer movieId,
            @RequestParam(name = PAGE_ATTRIBUTE, defaultValue = "0") int page,
            Model model) {

        boolean isFavorite = false;

        Authentication authentication = SecurityContextHolder.getContext().getAuthentication();
        Integer currentUserId = userService.getCurrentUserId(authentication.getName());

        FavoriteEntity fav = favoriteService.findByUserIdAndMovieId(currentUserId, movieId);

        if (fav == null) {
            isFavorite = false;
        } else {
            isFavorite = true;
        }

        if (isFavorite == false) {

            UserEntity user = userService.get(currentUserId);
            MovieEntity movie = movieService.get(movieId);

            FavoriteEntity newFav = new FavoriteEntity(null, user, movie);

            favoriteService.create(newFav);

        } else {

            FavoriteEntity delFav = favoriteService.findByUserIdAndMovieId(currentUserId, movieId);

            favoriteService.delete(delFav.getId());
        }

        model.addAttribute(PAGE_ATTRIBUTE, page);

        return Constants.REDIRECT_VIEW + URL;
    }

    @PostMapping("/changeViewStatus/{id}")
    public String changeViewStatus(@PathVariable(name = "id") Integer movieId,
            @RequestParam(name = PAGE_ATTRIBUTE, defaultValue = "0") int page,
            Model model) {

        boolean isViewed = false;

        Authentication authentication = SecurityContextHolder.getContext().getAuthentication();
        Integer currentUserId = userService.getCurrentUserId(authentication.getName());

        ViewedEntity view = viewedServise.findByUserIdAndMovieId(currentUserId, movieId);

        if (view == null) {
            isViewed = false;
        } else {
            isViewed = true;
        }

        if (isViewed == false) {

            UserEntity user = userService.get(currentUserId);
            MovieEntity movie = movieService.get(movieId);

            ViewedEntity newView = new ViewedEntity(null, user, movie);

            viewedServise.create(newView);

        } else {

            ViewedEntity delView = viewedServise.findByUserIdAndMovieId(currentUserId, movieId);

            viewedServise.delete(delView.getId());
        }

        model.addAttribute(PAGE_ATTRIBUTE, page);

        return Constants.REDIRECT_VIEW + URL;
    }
}
